#include "mgos.h"
#include "mgos_i2c.h"
#include "mgos_ina3221.h"
#include "mgos_ina219.h"
#include "mgos_ina226.h"
#include <fcntl.h>
#include <math.h>
#include <sys/ioctl.h>

#define I2CBUSNR    6

bool i2c_dumpregs(struct mgos_i2c *i2c, uint8_t i2caddr);
void i2c_scanner(struct mgos_i2c *i2c, bool dumpregs);

void i2c_scanner(struct mgos_i2c *i2c, bool dumpregs) {
  int i;

  if (!i2c) {
    LOG(LL_ERROR, ("No global I2C bus configured"));
    return;
  }

  for (i = 0x3; i < 0x77; i++) {
    bool ret;
    ret = mgos_i2c_read(i2c, i, NULL, 0, true);
    if (ret) {
      LOG(LL_INFO, ("I2C Address 0x%02x %s", i, ret ? "true" : "false"));
      if (dumpregs) i2c_dumpregs(i2c, i);
    }
  }
}

bool i2c_dumpregs(struct mgos_i2c *i2c, uint8_t i2caddr) {
  uint16_t reg;
  int      value;

  for (reg = 0; reg < 256; reg++) {
    value = mgos_i2c_read_reg_b(i2c, i2caddr, reg);
    if (value < 0) {
      printf(" XX");
    }else {
      printf(" %02x", value);
    }
    if (reg % 16 == 15) {
      printf("\n");
    }
  }
  printf("\n");
  return true;
}

static bool do_ina3221(struct mgos_ina3221 *sensor) {

  if (!sensor)
    return false;

  for (uint8_t i=1; i<4; i++) {
    float bus, shunt, current, res;
    mgos_ina3221_get_bus_voltage(sensor, i, &bus);
    mgos_ina3221_get_shunt_resistance(sensor, i, &res);
    mgos_ina3221_get_shunt_voltage(sensor, i, &shunt);
    mgos_ina3221_get_current(sensor, i, &current);
    LOG(LL_INFO, ("Chan[%u]: Vbus=%.3f V Vshunt=%.0f uV Rshunt=%.3f Ohm Ishunt=%.1f mA", i, bus, shunt*10e6, res, current*10e3));
  }

  return true;
}

static bool do_ina219(struct mgos_ina219 *sensor) {
  float bus, shunt, current, res;

  if (!sensor)
    return false;

  mgos_ina219_get_bus_voltage(sensor, &bus);
  mgos_ina219_get_shunt_resistance(sensor, &res);
  mgos_ina219_get_shunt_voltage(sensor, &shunt);
  mgos_ina219_get_current(sensor, &current);
  LOG(LL_INFO, ("Vbus=%.3f V Vshunt=%.0f uV Rshunt=%.3f Ohm Ishunt=%.1f mA", bus, shunt*10e6, res, current*10e3));

  return true;
}

static bool do_ina226(struct mgos_ina226 *sensor) {
  float bus, shunt, current, res;

  if (!sensor)
    return false;

  mgos_ina226_get_bus_voltage(sensor, &bus);
  mgos_ina226_get_shunt_resistance(sensor, &res);
  mgos_ina226_get_shunt_voltage(sensor, &shunt);
  mgos_ina226_get_current(sensor, &current);
  LOG(LL_INFO, ("Vbus=%.3f V Vshunt=%.0f uV Rshunt=%.3f Ohm Ishunt=%.1f mA", bus, shunt*10e6, res, current*10e3));

  return true;
}

int main(int argc, char **argv, char **environ) {
  struct mgos_i2c *    i2c = NULL;
  struct mgos_ina3221 *ina3221 = NULL;
  struct mgos_ina219 *ina219 = NULL;
  struct mgos_ina226 *ina226 = NULL;

  if (!mgos_i2c_open(I2CBUSNR)) {
    LOG(LL_ERROR, ("Cannot open I2C bus %u", I2CBUSNR));
    return -1;
  }
  if (!(i2c = mgos_i2c_get_global())) {
    LOG(LL_ERROR, ("Cannot open I2C bus"));
    return -2;
  }

  // i2c_scanner(i2c, true);

  /*
  if (!(ina3221 = mgos_ina3221_create(i2c, 0x40))) {
    LOG(LL_ERROR, ("Cannot create INA3221 device"));
    return -3;
  }
  */
  if (!(ina219 = mgos_ina219_create(i2c, 0x40))) {
    LOG(LL_ERROR, ("Cannot create INA219 device"));
    return -3;
  }
  /*
  if (!(ina226 = mgos_ina226_create(i2c, 0x40))) {
    LOG(LL_ERROR, ("Cannot create INA226 device"));
    return -3;
  }
  */

  for (;;) {
    do_ina3221(ina3221);
    do_ina219(ina219);
    do_ina226(ina226);
    sleep(1);
  }

  mgos_ina3221_destroy(&ina3221);
  mgos_ina219_destroy(&ina219);
  mgos_ina226_destroy(&ina226);

  return 0;
  (void)argc;
  (void)argv;
  (void)environ;
}
